## PyTorch

依赖
```text
pip3 install -i https://pypi.tuna.tsinghua.edu.cn/simple torch==1.10.2
```

示例代码: 
```python
#!/usr/bin/python3
# -*- coding: utf-8 -*-
from itertools import chain
import json
import os
import sys

pwd = os.path.abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(pwd, '../../../'))

import torch
import torch.nn as nn

from project_settings import project_path
from smart.cv2.misc import show_image
from smart.python_speech_features.misc import wave2spectrum_image


def demo1():
    import time

    import numpy as np
    from scipy.io import wavfile

    # 加载模型
    seq2seq_encoder = torch.jit.load('seq2seq_encoder.pth')
    seq2vec_encoder = torch.jit.load('seq2vec_encoder.pth')
    classification_layer = torch.jit.load('classification_layer.pth')
    with open('index2token.json', 'r', encoding='utf-8') as f:
        index2token = json.load(f)

    # 读取文件
    filename = os.path.join(project_path, 'datasets/asr/early_media/66/66_wav/no_answer/330088866619939351_62ed2322-0ca9-4f96-a769-7435ecb7461c.wav')
    sample_rate, wave = wavfile.read(filename)
    print('length: {}'.format(len(wave)))

    max_wave_value = 32768.0
    wave = wave / max_wave_value

    array = wave2spectrum_image(
        wave,
        sample_rate=8000,
        xmax=10,
        xmin=-50,
        winlen=0.025,
        winstep=0.01,
        nfft=512,
        n_low_freq=100,
    )

    # show_image(array.T)

    time_begin = time.time()

    array = np.array([array], dtype=np.float32)
    array = torch.tensor(array, dtype=torch.float32)
    mask: torch.IntTensor = torch.ones(size=array.shape[:-1], device=array.device, dtype=torch.int32)

    array = seq2seq_encoder.forward(array, mask)
    print('array shape: {}'.format(array.shape))

    length = array.shape[-2]

    print('n ms each frame: {}'.format(len(wave) / sample_rate * 1000 / array.shape[1]))
    m_win_size = 50
    m_win_step = 25

    labels = list()
    idx = 0
    while True:
        begin = idx * m_win_step
        end = begin + m_win_size
        if end > length:
            break
        window = array[:, begin:end, :]

        window = seq2vec_encoder.forward(window)

        logits = classification_layer(window)
        probs = torch.nn.functional.softmax(logits, dim=-1)
        label_idx = probs.argmax(dim=-1).item()

        label_str = index2token[str(label_idx)]
        labels.append(label_str)
        idx += 1

    print(labels)
    cost = time.time() - time_begin
    print('\n')
    print('cost: {}'.format(cost))

    return


if __name__ == '__main__':
    demo1()

```
